import 'dart:io' show Platform;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class PlatformWidget extends StatelessWidget {
  PlatformWidget({this.ios, this.android});
  final Widget Function(BuildContext) ios;
  final Widget Function(BuildContext) android;
  @override
  Widget build(BuildContext context) {
    return Platform.isAndroid ? android(context) : ios(context);
  }
}

class PlatformTabBar extends StatelessWidget {
  PlatformTabBar({
    this.items,
    this.activeColor,
    this.inactiveColor,
    this.backgroundColor,
    this.currentIndex,
    this.onTap,
  });
  final List<BottomNavigationBarItem> items;
  final Color activeColor;
  final Color inactiveColor;
  final Color backgroundColor;
  final int currentIndex;
  final Function(int) onTap;
  @override
  Widget build(BuildContext context) {
    return PlatformWidget(
      ios: (context) => CupertinoTabBar(
        backgroundColor: backgroundColor,
        activeColor: activeColor,
        inactiveColor: inactiveColor,
        items: items,
        onTap: onTap,
        currentIndex: currentIndex,
      ),
      android: (context) => new Theme(
        data: Theme.of(context).copyWith(
          // sets the background color of the `BottomNavigationBar`
          canvasColor: backgroundColor,
          textTheme: Theme.of(context).textTheme.copyWith(
                caption: TextStyle(
                  color: inactiveColor,
                ),
              ),
        ), // sets the inactive color of the `BottomNavigationBar`
        child: new BottomNavigationBar(
          type: BottomNavigationBarType.fixed,
          fixedColor: activeColor,
          currentIndex: currentIndex,
          onTap: onTap,
          items: items,
        ),
      ),
    );
  }
}

class PlatformSwitch extends StatelessWidget {
  PlatformSwitch(
      {@required this.value, @required this.onChanged, this.activeColor});
  final bool value;
  final Function(bool) onChanged;
  final Color activeColor;
  @override
  Widget build(BuildContext context) {
    return PlatformWidget(
      ios: (context) => CupertinoSwitch(
        value: value,
        onChanged: onChanged,
        activeColor: activeColor,
      ),
      android: (context) => Switch(
        value: value,
        onChanged: onChanged,
        activeColor: activeColor,
      ),
    );
  }
}

class PlatformLoader extends StatelessWidget {
  const PlatformLoader({Key key, this.centered, this.color}) : super(key: key);
  final bool centered;
  final Color color;

  @override
  Widget build(BuildContext context) {
    final _widget = PlatformWidget(
      ios: (context) => CupertinoActivityIndicator(
        animating: true,
        radius: 15.0,
      ),
      android: (context) => CircularProgressIndicator(
          valueColor: new AlwaysStoppedAnimation<Color>(color)),
    );

    if (centered ?? false) {
      return Center(child: _widget);
    } else {
      return _widget;
    }
  }
}

class PlatformButton extends StatelessWidget {
  PlatformButton({
    Key key,
    @required this.child,
    @required this.onPressed,
    this.color,
    this.disabledColor,
    this.padding,
    this.borderRadius = const BorderRadius.all(Radius.zero),
  }) : super(key: key);
  final Widget child;
  final void Function() onPressed;
  final Color color, disabledColor;
  final EdgeInsetsGeometry padding;
  final BorderRadius borderRadius;
  @override
  Widget build(BuildContext context) {
    return PlatformWidget(
      ios: (ctx) => CupertinoButton(
        child: child,
        onPressed: onPressed,
        color: color,
        disabledColor: disabledColor,
        pressedOpacity: 0.9,
        borderRadius: borderRadius,
        padding: padding ?? Theme.of(context).buttonTheme.padding,
      ),
      android: (ctx) => FlatButton(
        child: child,
        onPressed: onPressed,
        color: color,
        textTheme: ButtonTextTheme.primary,
        disabledColor: disabledColor,
        padding: padding ?? Theme.of(context).buttonTheme.padding,
      ),
    );
  }
}

Route<T> platformRoute<T>({WidgetBuilder builder, RouteSettings settings}) {
  if (Platform.isIOS) {
    return CupertinoPageRoute(
      builder: builder,
      settings: settings,
    );
  } else {
    return MaterialPageRoute(
      builder: builder,
      settings: settings,
    );
  }
}

void showPlatformDialog(BuildContext context,
    {String title, String content, List<Widget> actions}) {
  final _title = title != null ? Text(title) : null;
  final _content = content != null ? Text(content) : null;
  showDialog(
      context: context,
      builder: (_) {
        if (Platform.isIOS) {
          return CupertinoAlertDialog(
            title: _title,
            content: _content,
            actions: actions ?? [],
          );
        } else {
          return AlertDialog(
            title: _title,
            content: _content,
            actions: actions ?? [],
          );
        }
      });
}

void showPlatformBottomSheet(BuildContext context, {List<Widget> actions}) {
  if (Platform.isIOS) {
    showCupertinoModalPopup(
        context: context,
        builder: (ctx) => CupertinoActionSheet(
              cancelButton: PlatformButton(
                key: Key('bottom-sheet-cancel-button'),
                child: Text('Cancel'),
                onPressed: () => Navigator.of(ctx).pop(),
              ),
              actions: actions ?? [],
            ));
  } else {
    showModalBottomSheet<void>(
      context: context,
      builder: (ctx) => Column(
        mainAxisSize: MainAxisSize.min,
        children: actions,
      ),
    );
  }
}
